@@ -18,9 +18,15 @@
 
 import re
 import string
-import types
 from offlineimap.ui import getglobalui
-quotere = re.compile('^("(?:[^"]|\\\\")*")')
+
+
+# find the first quote in a string
+quotere = re.compile(
+    r"""(?P<quote>"(?:\\"|[^"])*") # Quote, possibly containing encoded
+                                   # quotation mark
+        \s*(?P<rest>.*)$           # Whitespace & remainder of string""",
+    re.VERBOSE)
 
 def debug(*args):
     msg = []
@@ -29,24 +35,33 @@ def debug(*args):
     getglobalui().debug('imap', " ".join(msg))
 
 def dequote(string):
-    """Takes a string which may or may not be quoted and returns it, unquoted.
-    This function does NOT consider parenthised lists to be quoted.
-    """
+    """Takes string which may or may not be quoted and unquotes it.
 
-    if not (string[0] == '"' and string[-1] == '"'):
-        return string
-    string = string[1:-1]               # Strip off quotes.
-    string = string.replace('\\"', '"')
-    string = string.replace('\\\\', '\\')
-    debug("dequote() returning:", string)
+    It only considers double quotes. This function does NOT consider
+    parenthised lists to be quoted.
+    """
+    if string and string.startswith('"') and string.endswith('"'):
+        string = string[1:-1]  # Strip off the surrounding quotes.
+        string = string.replace('\\"', '"')
+        string = string.replace('\\\\', '\\')
     return string
 
 def flagsplit(string):
+    """Converts a string of IMAP flags to a list
+
+    :returns: E.g. '(\\Draft \\Deleted)' returns  ['\\Draft','\\Deleted'].
+        (FLAGS (\\Seen Old) UID 4807) returns
+        ['FLAGS,'(\\Seen Old)','UID', '4807']
+    """
     if string[0] != '(' or string[-1] != ')':
-        raise ValueError, "Passed string '%s' is not a flag list" % string
+        raise ValueError("Passed string '%s' is not a flag list" % string)
     return imapsplit(string[1:-1])
 
 def options2hash(list):
+    """convert list [1,2,3,4,5,6] to {1:2, 3:4, 5:6}"""
+    # effectively this does dict(zip(l[::2],l[1::2])), however
+    # measurements seemed to have indicated that the manual variant is
+    # faster for mosly small lists.
     retval = {}
     counter = 0
     while (counter < len(list)):
@@ -55,8 +70,12 @@ def options2hash(list):
     debug("options2hash returning:", retval)
     return retval
 
-def flags2hash(string):
-    return options2hash(flagsplit(string))
+def flags2hash(flags):
+    """Converts IMAP response string from eg IMAP4.fetch() to a hash.
+
+    E.g. '(FLAGS (\\Seen Old) UID 4807)' leads to
+    {'FLAGS': '(\\Seen Old)', 'UID': '4807'}"""
+    return options2hash(flagsplit(flags))
 
 def imapsplit(imapstring):
     """Takes a string from an IMAP conversation and returns a list containing
@@ -68,7 +87,7 @@ def imapsplit(imapstring):
 
     ['(\\HasNoChildren)', '"."', '"INBOX.Sent"']"""
 
-    if type(imapstring) != types.StringType:
+    if not isinstance(imapstring, basestring):
         debug("imapsplit() got a non-string input; working around.")
         # Sometimes, imaplib will throw us a tuple if the input
         # contains a literal.  See Python bug
@@ -106,26 +125,28 @@ def imapsplit(imapstring):
                 retval.extend(imapsplit(arg))
         debug("imapsplit() non-string: returning %s" % str(retval))
         return retval
-        
+
     workstr = imapstring.strip()
     retval = []
     while len(workstr):
+        # handle parenthized fragments (...()...)
         if workstr[0] == '(':
             rparenc = 1 # count of right parenthesis to match
             rpareni = 1 # position to examine
- 	    while rparenc: # Find the end of the group.
- 	    	if workstr[rpareni] == ')':  # end of a group
- 			rparenc -= 1
- 		elif workstr[rpareni] == '(':  # start of a group
- 			rparenc += 1
- 		rpareni += 1  # Move to next character.
+            while rparenc: # Find the end of the group.
+                if workstr[rpareni] == ')':  # end of a group
+                    rparenc -= 1
+                elif workstr[rpareni] == '(':  # start of a group
+                    rparenc += 1
+                rpareni += 1  # Move to next character.
             parenlist = workstr[0:rpareni]
             workstr = workstr[rpareni:].lstrip()
             retval.append(parenlist)
         elif workstr[0] == '"':
-            quotelist = quotere.search(workstr).group(1)
-            workstr = workstr[len(quotelist):].lstrip()
-            retval.append(quotelist)
+            # quoted fragments '"...\"..."'
+            m = quotere.match(workstr)
+            retval.append(m.group('quote'))
+            workstr = m.group('rest')
         else:
             splits = string.split(workstr, maxsplit = 1)
             splitslen = len(splits)
@@ -144,7 +165,7 @@ def imapsplit(imapstring):
                 # There was not even an unquoted word.
                 break
     return retval
-            
+
 flagmap = [('\\Seen', 'S'),
            ('\\Answered', 'R'),
            ('\\Flagged', 'F'),
@@ -152,54 +173,48 @@ flagmap = [('\\Seen', 'S'),
            ('\\Draft', 'D')]
 
 def flagsimap2maildir(flagstring):
-    retval = []
-    imapflaglist = [x.lower() for x in flagstring[1:-1].split()]
+    """Convert string '(\\Draft \\Deleted)' into a flags set(DR)"""
+    retval = set()
+    imapflaglist = flagstring[1:-1].split()
     for imapflag, maildirflag in flagmap:
-        if imapflag.lower() in imapflaglist:
-            retval.append(maildirflag)
-    retval.sort()
+        if imapflag in imapflaglist:
+            retval.add(maildirflag)
     return retval
 
 def flagsmaildir2imap(maildirflaglist):
+    """Convert set of flags ([DR]) into a string '(\\Deleted \\Draft)'"""
     retval = []
     for imapflag, maildirflag in flagmap:
         if maildirflag in maildirflaglist:
             retval.append(imapflag)
-    retval.sort()
-    return '(' + ' '.join(retval) + ')'
+    return '(' + ' '.join(sorted(retval)) + ')'
 
-def listjoin(list):
-    start = None
-    end = None
-    retval = []
+def uid_sequence(uidlist):
+    """Collapse UID lists into shorter sequence sets
 
-    def getlist(start, end):
+    [1,2,3,4,5,10,12,13] will return "1:5,10,12:13".  This function sorts
+    the list, and only collapses if subsequent entries form a range.
+    :returns: The collapsed UID list as string"""
+    def getrange(start, end):
         if start == end:
             return(str(start))
-        else:
-            return(str(start) + ":" + str(end))
-        
+        return "%s:%s" % (start, end)
 
-    for item in list:
-        if start == None:
-            # First item.
-            start = item
-            end = item
-        elif item == end + 1:
-            # An addition to the list.
-            end = item
-        else:
-            # Here on: starting a new list.
-            retval.append(getlist(start, end))
-            start = item
-            end = item
+    if not len(uidlist): return '' # Empty list, return
+    start, end = None, None
+    retval = []
+    # Force items to be longs and sort them
+    sorted_uids = sorted(map(int, uidlist))
 
-    if start != None:
-        retval.append(getlist(start, end))
+    for item in iter(sorted_uids):
+        item = int(item)
+        if start == None:     # First item
+            start, end = item, item
+        elif item == end + 1: # Next item in a range
+            end = item
+        else:                 # Starting a new range
+            retval.append(getrange(start, end))
+            start, end = item, item
 
+    retval.append(getrange(start, end)) # Add final range/item
     return ",".join(retval)
-
-
-
-            
-        
